@@ -221,7 +221,7 @@ render_scene = function(scene, width = 400, height = 400, fov = 20,
                         ambient_light = NULL, 
                         lookfrom = c(0,1,10), lookat = c(0,0,0), camera_up = c(0,1,0), 
                         aperture = 0.1, clamp_value = Inf,
-                        filename = NULL, backgroundhigh = "#80b4ff",backgroundlow = "#ffffff",
+                        filename = NULL, backgroundhigh = "#80b4ff", backgroundlow = "#ffffff",
                         shutteropen = 0.0, shutterclose = 1.0, focal_distance=NULL, ortho_dimensions = c(1,1),
                         tonemap ="gamma", bloom = TRUE, parallel = TRUE, bvh_type = "sah",
                         environment_light = NULL, rotate_env = 0, intensity_env = 1,
@@ -295,20 +295,22 @@ Left Mouse Click: Change Look At (new focal distance) | Right Mouse Click: Chang
                                   intensity_env = intensity_env,
                                   debug_channel = debug_channel, return_raw_array = return_raw_array,
                                   progress = progress, verbose = verbose, sample_dist = Inf)
-  print_time(verbose, "Pre-processed  scene")
+  print_time(verbose, "Pre-processed scene")
   
   camera_info = scene_list$camera_info
   scene_info = scene_list$scene_info
-  processed_scene = scene_list$scene
+  render_info = scene_list$render_info
+  processed_scene = scene_info$scene
   
   camera_info$preview = preview
   camera_info$interactive = interactive
   debug_channel = scene_info$debug_channel  # converted to numeric
   
-  #Pathrace Scene
+  #Pathtrace Scene
   rgb_mat = render_scene_rcpp(scene = processed_scene, 
                               camera_info = camera_info, 
-                              scene_info = scene_info) 
+                              scene_info = scene_info,
+                              render_info = render_info) 
   if(!is.null(attr(rgb_mat,"keyframes"))) {
     message("Saving camera keyframes: Call `get_saved_keyframes()` function to return them.")
     keyframes = do.call(rbind,lapply(attr(rgb_mat,"keyframes"),as.data.frame))
